import WildEmitter from 'wildemitter';

interface DesktopCapturerSource {
    /**
     * ID of the screen source.
     */
    readonly id: string;
    /**
     * Name of the screen source.
     */
    readonly name: string;
    /**
     * Thumbnail of the screen source. See [nativeImage](https://electronjs.org/docs/api/native-image#nativeimage) for supported types.
     */
    readonly thumbnail: any;
}

interface ClientConfig {
    /**
     * The channel profile.
     *
     * Agora Web SDK needs to know the application scenario to apply different optimization methods.
     *
     * Currently Agora Web SDK supports the following channel profiles:
     * - `"live"`: Sets the channel profile as live broadcast. Host and audience roles that can be set by calling the [[Client.setClientRole]] method.
     * The host sends and receives voice/video, while the audience can only receive voice/video.
     * - `"rtc"`: Sets the channel profile as communication. This is used in one-on-one calls or group calls, where all users in the channel can talk freely.
     *
     * **Note**
     *
     * If you need to communicate with Agora Native SDK, Agora recommends the following settings:
     * - For Native SDK v2.3.2 and later:
     *  * Set {@link mode} as `"rtc"` or `"live"` if the Native SDK uses the communication channel profile.
     *  * Set {@link mode} as `"live"` if the Native SDK uses the live broadcast channel profile.
     * - For Native SDK before v2.3.2, set mode as "live" regardless of which channel profile the Native SDK uses.
     *
     * **Note**
     *
     * The `"rtc"` mode supports the Agora Recording SDK 2.3.3 or later.
     */
    mode: 'live' | 'rtc';
    /**
     * The codec the Web browser uses for encoding.
     * - `"vp8"`: Sets the browser to use VP8 for encoding.
     * - `"vp9"`: Sets the browser to use VP9 for encoding.
     * - `"h264"`: Sets the browser to use H.264 for encoding.
     *
     * **Note**
     *
     * - Safari 12.1 or earlier does not support the VP8 codec.
     * - Codec support on mobile devices is a bit complex, see [Use Agora Web SDK on Mobile Devices](https://docs.agora.io/en/faq/web_on_mobile) for details.
     */
    codec: 'vp8' | 'h264' | 'h264,vp8';

    server: string;

    /**
     * 是否启用调试模式，调试模式会连接到测试服务器
     */
    debug: boolean;
}

interface VideoEncoderConfiguration {
    /**
     * Resolution of the video.
     *
     * We recommend using common resolutions, for example:
     *
     * - 480 &times; 360
     * - 640 &times; 480
     * - 960 &times; 720
     *
     */
    resolution?: {
        /** Width of the video. The value range is [1,10000]. */
        width: number;
        /** Height of the video. The value range is [1,10000]. */
        height: number;
    };
    /**
     * The video encoding frame rate (fps). The value range is [1, 10000].
     *
     * Set this parameter as a value range. The minimum is **the expected frame rate**, and the maximum is the upper limit of the adjusted frame rate when the browser does not support the expected frame rate.
     *
     * **Note**
     * - The actual encoding frame rate depends on the device, system, and browser.
     * - When the network conditions change, the browser adjusts the encoding frame rate automatically.
     */
    frameRate?: {
        /** The default frame rate. The SDK uses this value as the expected frame rate. We recommend setting this value between 5 and 30. */
        min: number;
        /** The upper limit of the frame rate when the browser does not support the frame rate specified by `min`. */
        max: number;
    };
    /**
     * The video bitrate (Kbps). The value range is [1,10000000].
     *
     * We recommend setting the bitrate between 100 Kbps and 5000 Kbps. You can refer to the table below and set your bitrate.
     *
     * [[include:VideoProfileDefinition.md]]
     */
    bitrate?: {
        /** The minimum bitrate. */
        min: number;
        /** The maximum bitrate. */
        max: number;
    };
}

interface LocalStreamStats {
    /** Bytes of the sent audio. */
    audioSendBytes: string;
    /** Packets of the sent audio. */
    audioSendPackets: string;
    /**
     * Number of lost packets of the sent audio.
     *
     * **Note**
     *
     * Safari and Firefox do not support this property.
     */
    audioSendPacketsLost: string;
    /** RTT (Round-Trip Time). */
    audioRtt: string;
    /** Bytes of the sent video. */
    videoSendBytes: string;
    /** Packets of the sent video. */
    videoSendPackets: string;
    /** Number of lost packets of the sent video. */
    videoSendPacketsLost: string;
    /** Frame rate of the sent video. */
    videoSendFrameRate: string;
    /** Resolution width of the sent video. */
    videoSendResolutionWidth?: string;
    /** Resolution height of the sent video. */
    videoSendResolutionHeight?: string;
    /** Delay in accessing the SD-RTN (ms). */
    accessDelay: string;
}

interface RemoteStreamStats {
    /** Bytes of the received audio. */
    audioReceiveBytes: string;
    /** Packets of the received audio. */
    audioReceivePackets: string;
    /** Number of lost packets of the received audio. */
    audioReceivePacketsLost: string;
    /** Bytes of the received video. */
    videoReceiveBytes: string;
    /** Packets of the received video. */
    videoReceivePackets: string;
    /** Number of lost packets of the received video. */
    videoReceivePacketsLost: string;
    /** Frame rate rof the received video. */
    videoReceiveFrameRate: string;
    /** Decode frame rate after the video is received. */
    videoReceiveDecodeFrameRate: string;
    /** Resolution width of the received video. */
    videoReceiveResolutionWidth?: string;
    /** Resolution height of the received video. */
    videoReceiveResolutionHeight?: string;
    /** Delay in accessing the SD-RTN (ms). */
    accessDelay: string;
    /**
     * End to end delay (ms).
     *
     * Delay from sending to receiving data.
     */
    endToEndDelay: string;
    /**
     * Delay in receiving the video (ms).
     *
     * Delay from sending to playing the video, only supported by Chrome for now.
     */
    videoReceiveDelay: number;
    /**
     * Delay in receiving the audio (ms).
     *
     * Delay from sending to playing the audio, only supported by Chrome for now.
     */
    audioReceiveDelay: number;
}

interface StreamSpec {
    /**
     * The stream ID.
     *
     * Please set the stream ID as the user ID, which can be retrieved from the callback of {@link Client.join}.
     */
    streamID?: number | string;
    /**
     * Marks whether this stream contains an audio track.
     */
    audio: boolean;
    /**
     * Marks whether this stream contains a video track.
     */
    video: | boolean
    | 'screen'
    | 'window'
    | 'lowres' /* 320x240 */
    | 'lowres-16:9' /* 320x180 */
    | 'stdres' /* 640x480，默认值 */
    | 'stdres-16:9' /* 640x360 */
    | 'hires'
    | 'hires-16:9'
    | 'hdres' /* 都是 1280x720，不能支持4:3 */
    | 'fhdres' /* 1920x1080 */
    | '4kres'; /* 3840x2160 */
    /**
     * Marks whether this stream contains a screen-sharing track. See [Share the Screen](../../../screensharing_web?platform=Web) for details.
     */
    screen?: boolean;
    /**
     * Marks whether to share the audio playback when sharing the screen.
     *
     * **Since**
     * <br>&emsp;&emsp;&emsp;*3.0.0*
     *
     * - `true`: Share the local audio playback when sharing the screen.
     * - `false`: (Default) Do not share the local audio playback when sharing the screen.
     *
     * To share the local audio playback when sharing the screen, ensure that you set {@link screen} as `true`. We recommend also setting {@link audio} as `false`. If both `screenAudio` and `audio` are set as `true`, the stream only contains the local audio playback.
     *
     * **Note**
     *
     * - This function supports only Chrome 74 or later on Windows.
     * - For the audio sharing to take effect, the user must check **Share audio** in the pop-up window when sharing the screen.
     */
    screenAudio?: boolean;
    /**
     * Specifies the audio source of the stream.
     */
    audioSource?: MediaStreamTrack;
    /**
     * Specifies the video source of the stream.
     *
     * **Note**
     *
     * If you use a video source created by the Canvas API, re-draw on the canvas every one second when the drawing is still to keep the video publishing.
     */
    videoSource?: MediaStreamTrack;
    /**
     * The camera device ID retrieved from the {@link getDevices} method.
     *
     * The retrieved ID is ASCII characters, and the string length is greater than or equals to 0 and less than 256 bytes.
     *
     * When the string length is 0, this property is ignored.
     */
    cameraId?: string;
    /**
     * The microphone device ID retrieved from the {@link getDevices} method.
     *
     * The retrieved ID is ASCII characters, and the string length is greater than or equals to 0 and less than 256 bytes.
     *
     * When the string length is 0, this property is ignored.
     */
    microphoneId?: string;
    /**
     * Sets using the front or rear camera
     *
     * You can set this parameter to use the front or rear camera on mobile devices:
     * - `"user"`: The front camera.
     * - `"environment"`: The rear camera.
     */
    facingMode?: string;
    /**
     * **Since**
     * <br>&emsp;&emsp;&emsp;*3.2.0*
     *
     * Sets the video transmission optimization strategy:
     * - `"motion"`: Prioritizes video smoothness. In most cases, the SDK does not reduce the frame rate, but may reduce the sending resolution.
     * - `"detail"`: Prioritizes clarity. In most cases, the SDK does not reduce the sending resolution, but may reduce the frame rate.
     *
     * If you leave this property empty, the SDK uses the default transmission optimization strategy:
     * - For a screen-sharing video stream (setting the `screen` property as `true` when calling {@link createStream}, the default transmission optimization strategy is to prioritizes clarity.
     * - For the other types of video streams, the SDK may reduce the frame rate or the sending resolution in poor network conditions.
     */
    optimizationMode?: string;
    /**
     * Marks whether to mirror the local video image of the publisher in the local preview.
     *
     * This setting does not take effect in screen-sharing streams.
     *
     * - `true`: (Default) Mirror the local video.
     * - `false`: Do not mirror the local video.
     *
     * Agora recommends enabling this function when using the front camera, and disabling it when using the rear camera.
     */
    mirror?: boolean;
    /**
     * The extension ID of the Chrome screen-sharing extension.
     *
     * ASCII characters only, and the string length must be greater than 0 and less than 256 bytes.
     *
     * Set this property if you use the Chrome screen-sharing extension. See [Chrome Extension for Screen Sharing](../../../chrome_screensharing_plugin?platform=Web) for details.
     *
     * **Note**
     *
     * Chrome 72 and later versions support screen sharing without the extension. You can leave `extensionId` as empty.
     *
     * If you set the `extensionId`, then you need to use the screen-sharing extension.
     */
    extensionId?: string;
    /**
     * The screen-sharing mode on the Firefox browser.
     *
     * If you are using the Firefox browser, setting this property specifies the screen-sharing mode:
     * - `"screen"`: (default) share the current screen
     * - `"application"`: share all windows of an App
     * - `"window"`: share a specified window of an App
     *
     * **Note**
     *
     * Firefox on Windows does not support the application mode.
     *
     * See [Screen Sharing on Firefox](../../../screensharing_web?platform=Web#a-name-ff-a-screen-sharing-on-firefox) for details.
     */
    mediaSource?: 'screen' | 'application' | 'window';
    /** Marks whether to enable audio processing. */
    audioProcessing?: {
        /**
         * Marks whether to enable audio gain control.
         *
         * The default value is `true` (enable). If you wish not to enable the audio gain control, set `AGC` as `false`.
         *
         * **Note**
         *
         * Safari does not support this setting.
         */
        AGC?: boolean;
        /**
         * Marks whether to enable acoustic echo cancellation.
         *
         * The default value is `true` (enable). If you wish not to enable the  acoustic echo cancellation, set `AEC` as `false`.
         *
         * **Note**
         *
         * Safari does not support this setting.
         */
        AEC?: boolean;
        /**
         * Marks whether to enable automatic noise suppression.
         *
         * The default value is `true` (enable). If you wish not to enable automatic noise suppression, set `ANS` as `false`.
         *
         * **Note**
         *
         * - Safari does not support this setting.
         * - Noise suppression is always enabled on Firefox. Setting `ANS` as `false` does not take effect on Firefox.
         */
        ANS?: boolean;
    };
}

interface LiveTranscoding {
    /**
     * The width of the video in pixels.
     *
     * A positive integer, the default value is 640.
     *
     * - When pushing video streams to the CDN, ensure that `width` is at least 64; otherwise, the Agora server adjusts the value to 64.
     * - When pushing audio streams to the CDN, set `width` and `height` as 0.
     */
    width?: number;
    /**
     * The height of the video in pixels.
     *
     * A positive integer. The default value is 360.
     *
     * - When pushing video streams to the CDN, ensure that `height` is at least 64; otherwise, the Agora server adjusts the value to 64.
     * - When pushing audio streams to the CDN, set `width` and `height` as 0.
     */
    height?: number;
    /**
     * The bitrate (Kbps) of the output video stream.
     *
     * A positive integer. The default value is 400 Kbps. The value range is [1,1000000].
     *
     * Set the bitrate according to the {@link AgoraRTC.VideoEncoderConfiguration.bitrate Video Profile Table}. If you set a bitrate beyond the proper range, the SDK automatically adapts it to a value within the range.
     */
    videoBitrate?: number;
    /**
     * The video frame rate (fps) of the CDN live stream.
     *
     * A positive integer. The default value is 15 fps. The value range is [1, 30]. The Agora server adjusts any value over 30 to 30.
     */
    videoFramerate?: number;
    /**
     * **DEPRECATED**
     *
     * Latency mode:
     * - true: Low latency with unassured quality.
     * - false: (Default）High latency with assured quality.
     */
    lowLatency?: boolean;
    /**
     * The audio sampling rate:
     *
     * - 32000: 32 kHz
     * - 44100: (Default) 44.1 kHz
     * - 48000: 48 kHz
     */
    audioSampleRate?: 32000 | 44100 | 48000;
    /**
     * The audio bitrate (Kbps) of the CDN live stream.
     *
     * A positive integer. The default value is 48, and the highest value is 128.
     */
    audioBitrate?: number;
    /**
     * The number of audio channels for the CDN live stream.
     *
     * Agora recommends choosing 1 (mono), or 2 (stereo) audio channels. Special players are required if you choose 3, 4, or 5.
     *
     * - 1: (Default) Mono
     * - 2: Stereo
     * - 3: Three audio channels
     * - 4: Four audio channels
     * - 5: Five audio channels
     */
    audioChannels?: 1 | 2 | 3 | 4 | 5;
    /** The video GOP in frames. The default value is 30 frames. The value range is [1,10000]. */
    videoGop?: number;
    /**
     * The video codec profile type.
     *
     * Set it as 66, 77, or 100 (default). If you set this parameter to any other value, Agora adjusts it to the default value 100.
     *
     * - 66: Baseline video codec profile. Generally used for video calls on mobile phones.
     * - 77: Main video codec profile. Generally used for mainstream electronics, such as MP4 players, portable video players, PSP, and iPads.
     * - 100: (Default) High video codec profile. Generally used for high-resolution broadcasts or television.
     */
    videoCodecProfile?: 66 | 77 | 100;
    /** The number of users; default value is 0. The maximum is 17. */
    // userCount?: number;
    /**
     * The background color in RGB hex.
     *
     * Value only. Do not include a preceding #. The default value is 0x000000. The value range is [0x000000, 0xffffff].
     */
    // backgroundColor?: number;
    /**
     * Manages the user layout configuration in the CDN live streaming.
     *
     * Agora supports a maximum of 17 transcoding users in a CDN streaming channel. See [[TranscodingUser]] for details.
     */
    // transcodingUsers: TranscodingUser[];
    /**
     * Watermark images for the CDN live stream.
     *
     * **Since**
     * <br>&emsp;&emsp;&emsp;*3.0.0*
     *
     * See [[TranscodingWatermark]] for details.
     */
    // images: TranscodingWatermark[];
}

interface LocalAudioStats {
    /** Energy level of the captured audio. */
    readonly RecordingLevel?: string;
    /** Energy level of the sent audio. */
    readonly SendLevel?: string;
    /** Sampling rate, in kHz. */
    readonly SamplingRate?: string;
    /** Bitrate of the sent audio, in Kbps. */
    readonly SendBitrate?: string;
    /**
     * Encoding type of the sent audio.
     *
     * - "1": Opus.
     * - "2": AAC.
     */
    readonly CodecType?: string;
    /**
     * Whether the audio is muted or not.
     *
     * - "1": Muted.
     * - "0": Unmuted.
     */
    readonly MuteState?: string;
}

type IAudioProfile =
    | 'speech_low_quality'
    | 'speech_standard'
    | 'music_standard'
    | 'standard_stereo'
    | 'high_quality'
    | 'high_quality_stereo';

interface LocalVideoStats {
    /** Bitrate of the local video set in [[setVideoProfile]]. */
    readonly TargetSendBitrate?: string;
    /** Frame rate of the sent video, in fps. */
    readonly SendFrameRate?: string;
    /** Bitrate of the sent video, in Kbps. */
    readonly SendBitrate?: string;
    /** Width of the sent video, in pixels. */
    readonly SendResolutionWidth?: string;
    /** Height of the sent video, in pixels. */
    readonly SendResolutionHeight?: string;
    /** Delay from capturing to encoding the local video, in ms. */
    readonly EncodeDelay?: string;
    /**
     * Whether the video is muted or not.
     *
     * - "1": Muted.
     * - "0": Unmuted.
     */
    readonly MuteState?: string;
    /** Width (pixels) of the captured video. */
    readonly CaptureResolutionWidth?: string;
    /** Height (pixels) of the captured video. */
    readonly CaptureResolutionHeight?: string;
    /** Frame rate of the captured video, in fps. */
    readonly CaptureFrameRate?: string;
    /** Total freeze time of the encoded video, in seconds. */
    readonly TotalFreezeTime?: string;
    /** Total duration of the published video, in seconds. */
    readonly TotalDuration?: string;
}

interface LocalAudioStatsMap {
    [uid: string]: LocalAudioStats;
}

interface LocalVideoStatsMap {
    [uid: string]: LocalVideoStats;
}

interface MediaDeviceInfo {
    /**
     * Device ID
     *
     * The unique ID of the device.
     *
     * **Note**
     *
     * On Chrome 81 or later, the device ID is only available after the user has granted permission to use the media device.
     */
    readonly deviceId: string;
    /**
     * Device Label
     *
     * Returns a `DOMString` that is a label describing this device (for example "External USB Webcam").
     *
     * **Note**
     *
     * For security reasons, the label field is always blank unless an active media stream exists or the user has granted persistent permission for media device access.
     *
     */
    readonly label: string;
    /**
     * Device Type
     *
     * Returns an enumerated value that is "videoinput", "audioinput" or "audiooutput".
     */
    readonly kind: string;
}

interface NetworkStats {
    /**
     * Network type.
     *
     * - "bluetooth": Bluetooth network.
     * - "cellular": Cellular network.
     * - "ethernet": Ethernet.
     * - "none": No network.
     * - "wifi": Wi-Fi.
     * - "wimax": WiMax.
     * - "other": Other network type.
     * - "unknown": Unknown network type.
     * - "UNSUPPORTED": The browser does not support getting the network type.
     */
    NetworkType: 'bluetooth' | 'cellular' | 'ethernet' | 'none' | 'wifi' | 'wimax' | 'other' | 'unknown' | 'UNSUPPORTED';
}

interface RemoteAudioStats {
    /**
     * End-to-end delay in ms.
     *
     * Delay from capturing to playing the audio.
     */
    readonly End2EndDelay?: string;
    /**
     * Transport delay in ms.
     *
     * Delay from sending to receiving the audio.
     */
    readonly TransportDelay?: string;
    /** Packet loss rate (%) of the remote audio. */
    readonly PacketLossRate?: string;
    /** Volume of the received audio. */
    readonly RecvLevel?: string;
    /** Bitrate of the received audio, in Kbps. */
    readonly RecvBitrate?: string;
    /**
     * Decoding type of the received audio.
     *
     * - "1": Opus.
     * - "2": AAC.
     */
    readonly CodecType?: string;
    /**
     * Whether the audio is muted or not.
     *
     * - "1": Muted.
     * - "0": Unmuted.
     */
    readonly MuteState?: string;
    /** Total freeze time of the received audio. */
    readonly TotalFreezeTime?: string;
    /** Total playing duration of the received audio. */
    readonly TotalPlayDuration?: string;
}

interface RemoteAudioStatsMap {
    [uid: string]: RemoteAudioStats;
}

interface RemoteVideoStats {
    /**
     * End-to-end delay in ms.
     *
     * Delay from capturing to playing the video.
     */
    readonly End2EndDelay?: string;
    /**
     * Transport delay in ms.
     *
     * Delay from sending to receiving the video.
     */
    readonly TransportDelay?: string;
    /** Packet loss rate (%) of the remote video. */
    readonly PacketLossRate?: string;
    /** Bitrate of the received video, in Kbps. */
    readonly RecvBitrate?: string;
    /** Resolution width of the received video, in pixels. */
    readonly RecvResolutionWidth?: string;
    /** Resolution height of the received video, in pixels. */
    readonly RecvResolutionHeight?: string;
    /** Rendering frame rate of the decoded video, in fps. */
    readonly RenderFrameRate?: string;
    /**
     * Whether the video is muted or not.
     *
     * - "1": Muted.
     * - "0": Unmuted.
     */
    readonly MuteState?: string;
    /** Total freeze time of the received video. */
    readonly TotalFreezeTime?: string;
    /** Total playing duration of the received video. */
    readonly TotalPlayDuration?: string;
    /** Width (pixels) of the rendered video */
    readonly RenderResolutionWidth?: string;
    /** Height (pixels) of the rendered video */
    readonly RenderResolutionHeight?: string;
}

interface RemoteVideoStatsMap {
    [uid: string]: RemoteVideoStats;
}

interface SessionStats {
    /** Call duration in seconds, represented by an aggregate value. */
    readonly Duration?: string;
    /**
     * Number of users in the channel.
     *
     * - rtc mode: The number of all users in the channel.
     * - live mode
     *  - If the local user is an audience: The number of hosts in the channel + 1.
     *  - If the user is a host: The number of hosts in the channel.
     */
    readonly UserCount?: string;
    /** Total number of bytes sent, represented by an aggregate value. */
    readonly SendBytes?: string;
    /** Total number of bytes received, represented by an aggregate value. */
    readonly RecvBytes?: string;
    /** Total sent bitrate of the stream, in Kbps, represented by an instantaneous value. */
    readonly SendBitrate?: string;
    /** Total received bitrate of the stream, in Kbps, represented by an instantaneous value. */
    readonly RecvBitrate?: string;
}

interface SystemStats {
    /** Battery level of the system (%). */
    BatteryLevel: string;
}

interface TransportStats {
    /** RTT (Round-Trip Time) between the SDK and the access node of the SD-RTN, in ms. */
    readonly RTT?: string;
    /**
     * Network type.
     *
     * - "bluetooth": Bluetooth network.
     * - "cellular": Cellular network.
     * - "ethernet": Ethernet.
     * - "none": No network.
     * - "wifi": Wi-Fi.
     * - "wimax": WiMax.
     * - "other": Other network type.
     * - "unknown": Unknown network type.
     * - "UNSUPPORTED": The browser does not support getting the network type.
     *
     * **Note**
     *
     * Chrome 61 or later is required for this function, and the compatibility is not guaranteed.
     * See [Network Information API](https://developer.mozilla.org/en-US/docs/Web/API/Network_Information_API) for details.
     */
    readonly NetworkType?:
    | 'bluetooth'
    | 'cellular'
    | 'ethernet'
    | 'none'
    | 'wifi'
    | 'wimax'
    | 'other'
    | 'unknown'
    | 'UNSUPPORTED';
    /** The estimated available bandwidth for sending the stream, in Kbps. */
    readonly OutgoingAvailableBandwidth?: string;
}

/**
 * Error messages of stream playback
 *
 * When [[Stream.play]] fails to play the stream, this interface provides detailed error messages.
 *
 * In most cases, you can prompt the user to resume the playback ([[Stream.resume]]) by a user gesture except when the {@link status} is "aborted".
 */
interface StreamPlayError {
    /**
     * The status of the player:
     * - "aborted": The player is removed before the stream is played successfully.
     * - "paused": The player is stopped.
     */
    readonly status: string;
    /**
     * The reason why the playback fails. Usually, this value is an event that triggers the playback failure. Possible values include the following:
     * - "stalled": The failure might be caused by the browser policy. See [stalled event](https://developer.mozilla.org/en-US/docs/Web/API/HTMLMediaElement/stalled_event).
     * - "pause": The stream playback might be paused by the user. See [pause event](https://developer.mozilla.org/en-US/docs/Web/API/HTMLMediaElement/pause_event).
     * - "suspend": The failure might be caused by the browser policy. See [suspend event](https://developer.mozilla.org/en-US/docs/Web/API/HTMLMediaElement/suspend_event).
     * - "canplay": Some browsers automatically stop the playback when the playback window is not displayed on the screen. See [canplay event](https://developer.mozilla.org/en-US/docs/Web/API/HTMLMediaElement/canplay_event).
     * - "error": The playback failure is usually caused by autoplay restriction.
     *
     * Due to the differences in browsers, `reason` may have different values for the same event.
     */
    readonly reason: string;
}

type IClientRole = 'host' | 'audience';

/**
 * Stream类，抽象类,作为LocalStream和RemoteStream的父类
 *
 */

/**
 * @classdesc 流对象
 */
declare abstract class Stream extends WildEmitter {
    protected _stream: MediaStream | null;
    protected _isPlaying: boolean;
    protected _streamId: string;
    protected _id: string;
    protected _elementId: string;
    protected _isLocal: boolean;
    protected audioEle: any;
    protected videoEle: any;
    /**
     * 初始化音视频对象
     * 该方法初始化本地创建的音视频流对象。
     * 如果调用失败，错误信息请参考 getUserMedia 异常。
     * 部分错误信息可能会在调用失败的回调函数中出现，例如：{type: "error",  msg: "NotAllowedError", info: "Permission denied"}。
     * @param {function} [onSuccess] - 可选；成功回调
     * @param {function} [onFailure] - 可选；失败回调 (err: { type: 'warning' | 'error'; msg: string; info?: string }) => void
     * msg 字段可能出现以下值：
     * - NotAllowedError: 用户拒绝授予对应的摄像头或麦克风权限。
     * - MEDIA_OPTION_INVALID: 摄像头被占用或者分辨率不支持（早期浏览器）
     * - DEVICES_NOT_FOUND: 没有找到设备
     * - NOT_SUPPORTED: 浏览器不支持获取获取摄像头和麦克风
     * - PERMISSION_DENIED: 浏览器禁用设备或者用户拒绝打开设备
     * - CONSTRAINT_NOT_SATISFIED: 配置参数不合法（早期浏览器）
     * - UNDEFINED: 未定义错误
     * @returns {void}
     * @example
     * init(function() {
     * console.log('local stream initialized');
     *   // publish the stream
     *   //……
     * }, function(err) {
     *   console.error("local stream init failed ", err);
     *   //error handling
     * });
     */
    abstract init(onSuccess?: () => void, onFailure?: (err: {
        type: 'warning' | 'error';
        msg: string;
        info?: string;
    }) => void): void;
    play(elementId: string, option?: {
        fit?: 'cover' | 'contain';
        muted?: boolean;
    }, callback?: (err: null | StreamPlayError) => void): any;
    private _handleVideoMediaEvents;
    private _handleAudioEvents;
    private _attachAudioMediaEvents;
    private _attachVideoMediaEvents;
    /**
     * 恢复音视频流播放
     * 该方法一般在调用 Stream.play 之后，播放失败时调用。通常播放失败是由于浏览器策略阻止导致的。
     * 该方法需用手势触发。详见处理浏览器的自动播放策略。
     * @fulfil {any} - 恢复成功
     * @reject {any} - 恢复失败
     */
    resume(): Promise<any>;
    /**
     * 停止音视频流
     * 调用该方法停止播放 `Stream.play` 播放的音视频流。
     * @returns {void}
     */
    stop(): void;
    /**
     * 返回音视频流当前是否在播放状态
     * @returns {boolean}
     * - true：该音视频流正在渲染或播放。
     * - false：该音视频流没有渲染。
     */
    isPlaying(): boolean;
    /**
     * 彻底关闭音视频流
     * 该方法关闭当前的流。调用该方法会取消摄像头和麦克风的访问权限。
     */
    abstract close(): void;
    /**
     * 启用音频轨道
     * 该方法启用音频轨道。对本地流启用音频轨道后远端会触发 client.on("unmute-audio") 回调。
     * @returns {boolean}
     * - true：启用音频轨道成功。
     * - false：启用音频轨道失败，可能的原因包括没有音频、流未初始化、音频轨道已经启用等。
     */
    unmuteAudio(): boolean;
    /**
     * 禁用音频轨道
     * 该方法禁用音频轨道。
     * 对于本地流，调用该方法会停止发送音频，远端会触发 Client.on("mute-audio") 回调。
     * 对于远端流，调用该方法仍然会接收音频，但是会停止播放。
     * 对于本地流，在 createStream 时将 audio 设置为 true 才可使用该方法。
     * @returns {boolean}
     * - true：禁用音频轨道成功。
     * - false：禁用音频轨道失败，可能的原因包括没有音频、流未初始化、音频轨道已经禁用等。
     */
    muteAudio(): boolean;
    /**
     * 启用视频轨道
     * 该方法启用视频轨道。对本地流启用视频轨道后远端会触发 Client.on("unmute-video") 回调。
     * Note
     * 对于本地创建的流，在 createStream 时将 video 设置为 true 才可使用该方法。
     * 视频轨道默认为开启状态。如果你调用了 muteVideo，可调用本方法启用视频。
     * @returns {boolean}
     * - true：启用视频轨道成功。
     * - false：启用视频轨道失败，可能的原因包括没有视频、流未初始化、视频轨道已经启用等。
     */
    unmuteVideo(): boolean;
    /**
     * 禁用视频轨道
     * 该方法禁用视频轨道。
     * 对于本地流，调用该方法会停止发送视频，远端会触发 client.on("mute-video") 回调。
     * 对于远端流，调用该方法仍然会接收视频，但是会停止播放。
     * 对于本地创建的流，在 createStream 时将 video 设置为 true 才可使用该方法。
     * @returns
     * - true：禁用视频轨道成功。
     * - false：禁用视频轨道失败，可能的原因包括没有视频、流未初始化、视频轨道已经禁用等。
     */
    muteVideo(): boolean;
    /**
     * 启用音频轨道
     * 请使用 unmuteAudio。
     * 该方法启用音频轨道。在 createStream 时将 audio 设置为 true 才可使用该方法。
     * 音频轨道默认为开启状态。如果你调用了 disableAudio，可调用本方法启用音频。
     * @deprecated since version 0.0.1
     * @returns {boolean}
     * - true：启用音频轨道成功。
     * - false：启用音频轨道失败，可能的原因包括没有音频、流未初始化、音频轨道已经启用等。
     */
    enableAudio(): boolean;
    disableAudio(): boolean;
    /**
     * 启用视频轨道
     * 请使用 unmuteVideo。
     * 该方法启用视频轨道。在 createStream 时将 video 设置为 true 才可使用该方法。
     * 视频轨道默认为开启状态。如果你调用了 disableVideo，可调用本方法启用视频。
     * @deprecated since version 0.0.1
     * @returns {boolean}
     * - true：启用视频轨道成功。
     * - false：启用视频轨道失败，可能的原因包括没有视频、流未初始化、视频轨道已经启用等。
     */
    enableVideo(): boolean;
    disableVideo(): boolean;
    isVideoOn(): boolean;
    isAudioOn(): boolean;
    /**
     * 获取音频 flag
     * 该方法仅对本地流有效。
     * @returns {boolean}
     * - true: 该音视频流对象中包含音频资源。
     * - false: 该音视频流对象中不包含音频资源。
     */
    hasAudio(): boolean;
    /**
     * 获取视频 flag
     * 该方法仅对本地流有效。
     * @returns {boolean}
     * - true: 该音视频流对象中包含视频资源。
     * - false: 该音视频流对象中不包含视频资源。
     */
    hasVideo(): boolean;
    /**
     * 获取当前音量
     * 该方法获取的是当前时刻的音量。如果你想表示本地或远端音量的变化，我们建议你使用 setTimeout 或者 setInterval 方法实时获取。
     * 该方法对没有音频数据的流是无效的，并且会产生警告。
     * 受浏览器策略影响，在 Chrome 70+ 和 Safari 浏览器上，该方法必须由用户手势触发，详见处理浏览器的自动播放策略。
     * @returns {number | undefined}
     * - 获取的音量值，取值范围 [0,1]。
     * @example
     * setInterval(function() {
     *   var audioLevel = stream.getAudioLevel();
     *
     *   }, 100)
     */
    abstract getAudioLevel(): number | undefined;
    /**
     * 获取音频轨道
     * 该方法获取音视频流中的音频轨道，可与 replaceTrack 搭配使用。
     * @returns {MediaStreamTrack | undefined}
     * - 如果音视频流中包含音频轨道，会以 MediaStreamTrack 对象返回。
     */
    getAudioTrack(): MediaStreamTrack | undefined;
    /**
     * 获取视频轨道
     * 该方法获取音视频流中的视频轨道，可与 replaceTrack 搭配使用。
     * @returns {MediaStreamTrack | undefined}
     * - 如果音视频流中包含视频轨道，会以 MediaStreamTrack 对象返回。
     */
    getVideoTrack(): MediaStreamTrack | undefined;
    /**
     * 获取音视频轨道
     * @returns {MediaStreamTrack[] | undefined}
     * - 如果音视频流存在且包含音频或视频流，会以 MediaStreamTrack数组返回
     */
    getTracks(): MediaStreamTrack[] | undefined;
    /**
     * 替换音视频轨道
     * 该方法可以替换本地音视频流中的音视频轨道。
     * 本地流发布后，可以通过该方法切换摄像头或者切换麦克风和播放的 mp3 等。
     * 新的音视频轨道可以通过 getUserMedia，MediaElement.captureStream 等方法获取。
     * 被替换的音视频轨道会被强制停止。
     * @param {MediaStreamTrack} track - 要替换的track
     * @param {function} [onSuccess] - 可选；成功回调；() => void;
     * @param {function} [onFailure]
     * - 可选；失败回调；(err: string) => void;
     * - 方法调用失败的回调。常见的错误如下：
     * - "INVALID_TRACK" 和 "INVALID_TRACK_TYPE": 新传入的 MediaStreamTrack 对象无法识别，请检查传入对象。
     * - "MEDIASTREAM_TRACK_NOT_FOUND": 找不到替换对象，比如在纯音频流上尝试替换一个视频的 Track。
     * - "NO_STREAM_FOUND": 找不到本地流对象，无法替换。
     * @example
     * - 支持 Chrome 65+，Safari 以及最新版 Firefox 浏览器。
     * - 部分移动设备上该方法可能不生效。
     * - Firefox 不支持在不同的麦克风之间切换音频轨道，可以支持切换麦克风和播放的音乐文件。
     * - Safari 部分外接音频设备不支持此方法。
     * - 订阅端无法知悉音视频轨道被替换。
     * - 推荐使用 switchDevice 方法来切换媒体输入设备。
     */
    abstract replaceTrack(track: MediaStreamTrack, onSuccess?: () => void, onFailure?: (err: string) => void): void;
    /**
     * 添加音视频轨道
     * 该方法将音视频轨道添加到 Stream。添加成功后，远端会触发 client.on("stream-updated") 回调。
     * 该方法不支持 Firefox 和 Safari。
     * 一个 Stream 对象最多只能包含一个音频轨道和一个视频轨道。
     * @param {MediaStreamTrack} track
     * - 要添加的媒体流
     * - 可以通过 mediaStream 方法获取音视频轨道。
     * @returns {void}
     * @example
     * var localStream = YMRTC.createStream({audio: true, video: false});
     * localStream.addTrack(anotherStream.getVideoTrack());
     */
    abstract addTrack(track: MediaStreamTrack): void;
    /**
     * 移除音视频轨道
     * 该方法将音视频轨道从 Stream 中移除。移除成功后，远端会触发 Client.on("stream-updated") 回调。
     * - 该方法不支持 Firefox 和 Safari。
     * - 如需同时更改音频和视频轨道，建议使用 replaceTrack 方法。
     * @param {MediaStreamTrack} track
     * - 可以通过 mediaStream 方法获取音视频轨道。
     * @returns {void}
     */
    abstract removeTrack(track: MediaStreamTrack): void;
    /**
     * 设置音频属性
     * 该方法设置本地流的音频属性，为可选项，且需在 Stream.init 之前调用。默认值为 "music_standard"。
     * @param {string} profile
     * - 'speech_low_quality'
     * - 'speech_standard'
     * - 'music_standard'
     * - 'standard_stereo'
     * - 'high_quality'
     * - 'high_quality_stereo';
     */
    abstract setAudioProfile(profile: IAudioProfile): void;
    setAudioVolume(volume: number): void;
    setAudioOutput(deviceId: string, onSuccess?: () => void, onFailure?: (err: string) => void): void;
    abstract switchDevice(type: 'audio' | 'video', deviceId: string, onSuccess?: () => void, onFailure?: (err: string) => void): void;
    /**
     * 设置视频属性
     * 该方法设置本地流的视频编码属性。每个属性对应一套视频参数，如分辨率、帧率、码率等。
     * 该方法一般在 Stream.init 之前调用，也可在 Stream.init 成功后调用本方法动态修改视频属性。
     * - 该方法仅可对本地流调用。
     * - 动态修改视频属性仅支持 Chrome 63 及以上版本和 Safari 11 及以上版本。
     * - 在部分 iOS 设备上动态修改视频属性可能会导致视频出现黑边。
     * - 请避免在发布流时调用该方法。
     * - 对于自采集流（通过设置 audioSource 和 videoSource 属性创建的音视频流），该方法设置的分辨率和帧率都不生效。
     * - 视频能否达到 1080p 以上的分辨率取决于设备的性能，在性能配备较低的设备上有可能无法实现。如果采用 720p 分辨率而设备性能跟不上，则有可能出现帧率过低的情况。
     * - 随着浏览器的升级，上表中列出的浏览器的分辨率支持可能并不完整，具体支持以实际情况为准。
     * - 部分浏览器的某些版本可能不完全支持上表中列出的分辨率，在这种情况下建议使用主流分辨率（即上表中 _1 后缀的分辨率）。
     * - Safari 浏览器不支持自定义视频帧率（默认为 30 fps）。如果你设置的视频帧率不是 30 fps，Safari 浏览器可能会修改或者拒绝你的设置。
     * - 由于设备和浏览器的限制，部分浏览器对设置的 Video Profile 不一定能全部适配。这种情况下浏览器会自动调整分辨率，计费也将按照实际分辨率计算。
     * - 由于设备和浏览器的限制，部分浏览器对设置的屏幕属性不一定能全部适配。这种情况下浏览器会自动调整分辨率，计费也将按照实际分辨率计算。
     * @param {string} profile - '视频属性，默认值为 "480p_1'
     * @returns {void}
     */
    abstract setVideoProfile(profile: string): void;
    abstract setVideoEncoderConfiguration(config: VideoEncoderConfiguration): void;
    /**
     * 设置屏幕属性
     * 该方法设置屏幕共享时屏幕的显示属性，必须在 Stream.init 之前调用。
     * - 该方法仅可对本地流调用。
     * @param profile
     */
    setScreenProfile(profile: string): void;
    abstract setVideoBitRate(bitrate: number | 'unlimited'): void;
    /**
     * 获取音视频流 ID
     * 该方法可以获取音视频流 ID。
     * @returns {string} - 流id
     *
     */
    getId(): string;
    getStreamId(): string;
    getUserId(): string;
    abstract getStats(callback: (stats: LocalStreamStats | RemoteStreamStats) => void): void;
    _setMediaStream(stream: MediaStream): void;
    _getMediaStream(): MediaStream | null;
    _setUserId(uid: string): void;
}

/**
 * @typedef {Object} LiveTranscoding
 * @desc 管理 CDN 直播推流转码的接口
 * 调用 `setLiveTranscoding` 时可以通过该接口设置直播推流。
 * @param {number} [width] - 视频的宽
 * @param {number} [height] - 视频的高
 * @param {number} [videoBitrate] - CDN直播流的视频码率（Kbps）
 * @param {number} [videoFramerate] - CDN直播流的帧率（fps）
 * @param {number} [audioSampleRate] - 音频采样率 32000： 32kHz, 44100: (默认） 44.1kHz, 48000: 48kHz
 * @param {number} [audioBitrate] - CDN直播流的音频码率（Kbps）
 * @param {number} [audioChannels] - CDN直播流的声道数. 1: (默认)Mono, 2: 立体, 3, 4, 5声道
 */
/**
 * @typedef {Object} NetworkType
 * @desc 网络类型
 * - "bluetooth"：蓝牙网络
 * - "cellular"：蜂窝移动数据网络
 * - "ethernet"：以太网
 * - "none"：没有网络
 * - "wifi"：Wi-Fi
 * - "wimax"：WiMax
 * - "other"：其他网络类型
 * - "unknown"：未知网络类型
 * - "UNSUPPORTED"：浏览器不支持获取网络类型
 *
 * 该功能仅支持 Chrome 61 及以上版本，且无法保证兼容性
 */
/**
 * @classdesc Client对象
 */
declare class RTCClient extends WildEmitter {
    private _uid;
    private readonly _commonData;
    private readonly _janusManager;
    private _publisher;
    private _room;
    private _role;
    private readonly _subscriberControl;
    private readonly _mode;
    private _liveTranscoding;
    private _subscribeMap;
    private _publishMap;
    get uid(): string;
    /**
     * 私有
     * @param {ClientConfig} config - 构造参数
     */
    constructor(config: ClientConfig);
    /**
     * 初始化客户端对象
     * 使用该方法初始化客户端对象。
     * @param {string} appkey - 项目appkey, 在https://youme.im 官网可查
     * @param {number} appid - 项目appid, 在https://youme.im 官网可查
     * @param {function} [onSuccess] - 可选，成功回调, () => void;
     * @param {function} [onFaialure] - 可选，失败回调, (err: string) => void
     * @returns {void}
     *
     * @example
     * client.init('appkey', () => {}, (err) => {})
     */
    init(appkey: string, appid: number, onSuccess?: () => void, onFaialure?: (err: string) => void): void;
    /**
     * 加入频道
     * 调用该方法加入频道时，本地会触发 client.on("connected") 和 client.on("connection-state-change") 回调；
     * 通信场景下的用户和直播场景下的主播加入频道后，远端会触发 client.on("peer-online") 回调；
     * tokenV3计算方式：token=sha1(apikey+appkey+roomid+userid+timestamp)
     * @param {string} token - token，非tokenV3输入空串
     * @param {string} channel - 频道
     * @param {string | null} uid - 用户id，不填则sdk创建随机的uid
     * @param {number} timestamp - tokenV3时间戳，非tokenV3输入0
     * @param {string} [optionalInfo=''] - 可选信息
     * @param {function} [onSuccess] - 可选；成功回调, (uid: number | string) => void
     * @param {function} [onFailure] - 可选；失败回调, (err: string) => void
     * @return {void}
     *
     * @example
     * client.join('token', 'channel', 'uid', '', (uid) => {}, (err) => {})
     */
    join(token: string, channel: string, uid: string | null, timestamp: number, optionalInfo?: string, onSuccess?: (uid: number | string) => void, onFailure?: (err: string) => void): void;
    /**
     * 踢人
     * 调用该方法踢人时，本端和远端都会触发 client.on("kicked-other-user")回调；
     * 被踢的人会触发 client.on("kicked-user") 回调
     * @param {string} kicked_user - 被踢的用户
     * @param {string} is_admin - 是否是管理员踢人
     * @param {string} kick_time - 被踢的时长
     * @param {function} [onSuccess] - 可选；成功回调, (uid: number | string) => void
     * @param {function} [onFailure] - 可选；失败回调, (err: string) => void
     * @return {void}
     *
     * @example
     * client.kick('kicked_user', 'is_admin', 'kick_time', (uid) => {}, (err) => {})
     */
    kickout(kicked_user: string, is_admin: number, kick_time: number, onSuccess?: (uid: number | string) => void, onFailure?: (err: string) => void): void;
    /**
     * 离开频道
     * 调用该方法离开频道时，本地会触发 Client.on("connection-state-change") 回调；
     * 通信场景下的用户和直播场景下的主播离开频道后，远端会触发 Client.on("peer-leave") 回调。
     * @param {function} [onSuccess] - 可选；成功回调；() => void;
     * @param {function} [onFailure] - 可选；失败回调；(err: string) => void;
     * @returns {void}
     *
     * @example
     * client.leave(() => {}, (err) => {});
     */
    leave(onSuccess?: () => void, onFailure?: (err: string) => void): void;
    /**
     * 该方法创建并返回音视频流对象。
     *
     * @param {StreamSpec} spec - 定义音视频流对象的属性，详见 {StreamSpec}。
     * @return {Stream} - 流对象
     */
    createStream(spec: StreamSpec): Stream;
    /**
     * 发布本地音视频流
     * 发布音视频流之后，本地会触发 client.on("stream-published") 回调；远端会触发 client.on("stream-added") 回调。
     * @param {Stream} stream - 要被发布的本地音视频流对象
     * @param {function} [onSuccess] - 可选；成功回调；() => void;
     * @param {function} onFailure - 可选；失败回调；(err: string) => void
     * @returns {void}
     * @example
     * client.publish(stream, (err) => {});
     */
    publish(stream: Stream, onSuccess?: () => void, onFailure?: (err: string) => void): void;
    /**
     * 取消发布本地音视频流
     * 该方法取消发布本地音视频流。
     * 取消发布音视频流之后，本地会触发 client.on("stream-unpublished") 回调，远端会触发 client.on("stream-removed") 回调。
     * @param {Stream} stream - 要取消发布的本地音视频流对象
     * @param {function} [onSuccess] - 可选；成功回调；() => void;
     * @param {function} [onFailure] - 可选；失败回调；(err: string) => void;
     * @returns {void}
     * @example
     * client.unpublish(stream, (err) => {});
     */
    unpublish(stream: Stream, onSuccess?: () => void, onFailure?: (err: string) => void): void;
    /**
     * @typedef {Object} SubscribeOptions
     * @desc subscribe方法需要的options参数
     * @param {boolean} [video] - 可选；是否订阅视频
     * @param {boolean} [audio] - 可选；是否订阅音频
     */
    /**
     * 订阅远端音视频流
     * 该方法从服务器端接收远端音视频流。
     * 订阅远端音视频流之后，本地会触发 client.on("stream-subscribed") 回调。
     * 如果订阅流中包含音频，还会触发 client.on("first-audio-frame-decode") 回调；- 待实现
     * 如果订阅流中包含视频，还会触发 client.on("first-video-frame-decode") 回调。- 待实现
     * @param {Stream} stream - 要取消订阅的远端音视频流
     * @param {SubscribeOptions} [options] - 可选；选项参数；默认订阅音视频流
     * @param {function} [onSuccess] - 可选；成功回调；() => void;
     * @param {function} [onFailure] - 可选；失败回调；(err) => void;
     *
     * @example
     * // 基本用法
     * client.subscribe(stream, (err) => {});
     *
     * @example
     * // 高级用法
     * client.subscribe(stream, { video: true, audio: true }, (err) => {});
     */
    subscribe(stream: Stream, options?: {
        video?: boolean;
        audio?: boolean;
    }, onSuccess?: () => void, onFailure?: (err: string) => void, force?: boolean): void;
    /**
     * 取消订阅远端音视频流
     * 该方法取消接收远端音视频流。
     * @param {Stream} stream - 要取消订阅的远端流
     * @param {function} [onSuccess] - 可选；成功回调；() => void;
     * @param {function} [onFailure] - 可选；失败回调；(err: string) => void;
     * 方法调用失败时执行的回调函数，常见的错误如下：
     * - "INVALID_REMOTE_STREAM": 传入的 stream 对象格式非法。
     * - "INVALID_OPERATION": 当前不在频道中，可能是没有加入或者是网络波动导致暂时断开连接。
     * - "NO_SUCH_REMOTE_STREAM": 没有找到要取消订阅的远端流。
     * - "UNSUBSCRIBE_STREAM_FAILED": 取消订阅失败，一般是取消订阅时连接断开。
     * @returns {void}
     * @example
     * client.unsubscribe(stream, ()=>{},(err) => {});
     */
    unsubscribe(stream: Stream, onSuccess?: () => void, onFailure?: (err: string) => void): void;
    private _delayPub;
    rePublishAll(): void;
    reSubscribeAll(): void;
    /**
     * 设置用户角色
     * 本方法仅适用于直播场景。
     * 直播场景下，可以调用本方法设置用户角色。
     * 在加入频道前，用户可以通过本方法设置自己的角色。
     * 在加入频道后，用户可以通过本方法切换角色：
     * 用户在调用 publish 的时候会自动切换成 host 角色，调用 unpublish 的时候会自动切换为 audience 角色。
     * 如果已经调用了 publish，调用本方法将用户角色设置为 audience，会自动进行 unpublish。
     * 如果你在加入频道后调用该方法切换用户角色，切换成功后，本地会触发 client.on("client-role-changed") 回调；远端会触发 client.on("peer-online") 或者 client.on("peer-leave") 回调。
     * @param {string} role - 角色, 'host'或'audience'
     * @param {function} [callback] - 可选；回调；(err?: string | null) => void;
     * @returns {void}
     * @example
     * client.setClientRole('host', (err) => {
     *     if (err) {
     *         // 失败
     *     } else {
     *         // 成功
     *     }
     * });
     */
    setClientRole(role: IClientRole, callback?: (err?: string | null) => void): void;
    /**
    * 发送自定义消息，支持和原生SDK的sendMessage 接口互通
    *
    * 接收者通过 client.on("custom-message") 接收
    * user_id 为可选参数，用于指定特定的接收userid，如果不指定就是频道内广播
    */
    sendMessage(msg: string, user_id?: string): Promise<void>;
    setOtherSpeakerMute(user_id: string, isOn: boolean): Promise<void>;
    setOtherMicMute(user_id: string, isOn: boolean): Promise<void>;
    setLowStreamParameter(param: {
        width?: number;
        height?: number;
        framerate?: number;
        bitrate?: number;
    }): void;
    setRemoteVideoStreamType(stream: Stream, streamType: 0 | 1): void;
    enableAudioVolumeIndicator(): void;
    configPublisher(width: number, height: number, framerate: number, bitrate: number, publisherUrl: string): void;
    /**
     * 新建直播流
     * 该方法新建直播流
     * 推流开始后本地会触发 client.on("liveStreamingStarted") 回调。如果推流失败会触发 client.on("liveStreamingFailed") 回调。
     * @param url - 接收推流的地址例如: rtmp://xxxxxxxxx
     * @returns {void}
     * @example
     * client.setLiveTranscoding(<coding>);
     * client.startLiveStreaming(<url>)
     */
    startLiveStreaming(url: string): void;
    /**
     * 设置直播推流转码
     * 调用该方法更新转码设置后本地会触发 client.on("liveTranscodingUpdated") 回调。
     * @param {LiveTranscoding} coding - 转码设置
     * @returns {void}
     */
    setLiveTranscoding(coding: LiveTranscoding): void;
    /**
     * 删除直播流
     * 该方法停止并删除直播流。推流停止后本地会触发 client.on("liveStreamingStopped") 回调。
     * @param {string} url - 要停止的流url
     * @returns {void}
     */
    stopLiveStreaming(url: string): void;
    /**
     * 获取系统网络信息
     *
     * @param {function} callback - 回调 (stats: NetworkStats) => void
     * @deprecated
     * @example
     * client.getNetworkStats((stats) => {
     *   console.log(`Current network type: ${stats.NetworkType}`);
     * });
     */
    getNetworkStats(callback: (stats: NetworkStats) => void): void;
    getSystemStats(callback: (stats: SystemStats) => void): void;
    /**
     * 枚举音频输入设备
     * 该方法枚举可用的音频输入设备，比如麦克风。
     * 调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的音频输入设备。
     * @param {function} callback - (devices: MediaDeviceInfo[]): void
     * @returns {void}
     * @example
     * client.getRecordingDevices((devices) => {});
     */
    getRecordingDevices(callback: (devices: MediaDeviceInfo[]) => void): void;
    getRemoteAudioStats(callback: (stats: RemoteAudioStatsMap) => void): void;
    /**
     * 枚举音频输出设备
     *
     * 该方法枚举可用的音频输出设备，比如扬声器。
     *
     * 调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的音频输出设备。
     *
     * @param {function} callback - 回调 (devices: MediaDeviceInfo[]) => void
     * @returns {void}
     */
    getPlayoutDevices(callback: (devices: MediaDeviceInfo[]) => void): void;
    /**
     * getCameras方法回调函数类型
     * @callback getCameras
     * @param {MediaDeviceInfo[]} devices - 视频输入设备列表
     * @returns {void}
     */
    /**
     * 枚举视频输入设备
     * 该方法枚举可用的视频输入设备，比如摄像头。
     * 调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的视频输入设备。
     * @param {getCameras} callback
     * @returns {void}
     */
    getCameras(callback: (devices: MediaDeviceInfo[]) => void): void;
    getLocalAudioStats(callback: (stats: LocalAudioStatsMap) => void): void;
    getRemoteVideoStats(callback: (stats: RemoteVideoStatsMap) => void): void;
    getLocalVideoStats(callback: (stats: LocalVideoStatsMap) => void): void;
    getTransportStats(callback: (stats: TransportStats) => void): void;
    getSessionStats(callback: (stats: SessionStats) => void): void;
    getConnectionState(): string;
    isLive(): boolean;
    /**
     * 设置断网重连后是否重新发布及订阅
     * @param {0 | 1} flag - 是否重新发布及订阅标志，默认是1，如果设置为0则需要业务自己去做重连操作
     * @returns {void}
     * @example
     * client.setRePubAndSub(0);
     */
    setRePubAndSub(flag: 0 | 1): void;
    /**
     * 该回调通知应用远端音视频流已添加。
     * 如果用户加入频道时频道内已经有其他用户，也会收到该回调报告已经存在的远端流。
     * @event RTCClient#event:stream-added
     * @property {string} type - 事件类型, 为 `stream-added`
     * @property {string} msg - 消息
     * @property {string} reason - 原因
     * @property {Stream} stream - 远端流
     */
    /**
     * 该回调通知应用已删除远端音视频流，即对方调用了 `client.unpublish`。
     * @event RTCClient#event:stream-removed
     * @property {string} type - 事件类型，为 `stream-removed`
     * @property {string} msg - 消息
     * @property {string} reason - 原因
     * @property {Stream} stream - 远端流
     */
    /**
     * 该回调提示有远端用户/主播加入频道。
     * - 通信场景下，该回调提示有远端用户加入了频道，并返回新加入用户的 ID；如果加入之前，已经有其他用户在频道中了，新加入的用户也会收到这些已有用户加入频道的回调。
     * - 直播场景下，该回调提示有主播加入了频道，并返回该主播的 ID。如果在加入之前，已经有主播在频道中了，新加入的用户也会收到已有主播加入频道的回调。建议连麦主播不超过 20
     *
     * 该回调在如下情况下会被触发：
     * - 通信场景的远端用户/直播场景的远端主播调用 client.join 方法加入频道。
     * - 直播场景的远端观众加入频道后调用 client.setClientRole 将用户角色改变为主播。
     * @event RTCClient#event:peer-online
     * @type {object}
     * @property {string} uid - 用户ID
     */
    /**
     * 该回调通知应用有远端用户离线。
     * 在直播场景中，只有角色为主播的用户会触发该回调。
     *
     * 离线包括以下情况：
     * - 用户离开频道
     *    - 正常离开：用户会收到类似“再见”的消息，接收此消息后，判断对方离开频道。
     *    - 超时掉线：在 20 秒内，用户没有收到对方的任何数据包，则判定为对方掉线。在网络较差的情况下，有可能会误报。建议使用信令系统来做可靠的掉线检测。
     * - 用户角色从主播变为观众
     * @event RTCClient#event:peer-leave
     * @type {object} evt
     * @property {string} reason
     * 用户离线原因
     * - "Quit": 用户调用了 Client.leave 主动离开频道。
     * - "BecomeAudience": 用户角色从主播切换为观众。
     *
     * @example
     * client.on("peer-leave", function(evt) {
     *   var uid = evt.uid;
     *   var reason = evt.reason;
     *   console.log("remote user left ", uid, "reason: ", reason);
     * });
     */
    /**
     * 远端用户求情把麦克风声音开启，收到该请求需要自己调用接口取消麦克风静音。
     * @event RTCClient#event:request-unmute-audio
     * @type {object} evt
     * @property {string} uid - 用户id
     *
     * @example
     * client.on("request-unmute-audio", function(evt) {
     *    var uid = evt.uid;
     *    console.log(uid + "request unmute my audio" );
     * });
     */
    /**
     * 远端用户求情把麦克风声音关闭，收到该请求需要自己调用接口静音麦克风。
     * @event RTCClient#event:request-mute-audio
     * @type {object} evt
     * @property {string} uid - 用户id
     * @example
     * client.on("request-mute-audio", function(evt) {
     *    var uid = evt.uid;
     *    console.log(uid + "request mute my audio");
     * });
     */
    /**
     * 该回调通知应用对方用户在语音通话中将自己的声音打开。
     * @event RTCClient#event:unmute-audio
     * @type {object} evt
     * @property {string} uid - 用户id
     *
     * @example
     * client.on("unmute-audio", function(evt) {
     *    var uid = evt.uid;
     *    console.log("unmute audio:" + uid);
     * });
     */
    /**
     * 该回调通知应用对方用户在语音通话中将自己的声音关闭。
     * @event RTCClient#event:mute-audio
     * @type {object} evt
     * @property {string} uid - 用户id
     * @example
     * client.on("mute-audio", function(evt) {
     *    var uid = evt.uid;
     *    console.log("mute audio:" + uid);
     * });
     */
    /**
     * 该回调通知应用对方用户在语音通话中将自己的视频打开。
     * @event RTCClient#event:unmute-video
     * @type {object} evt
     * @property {string} uid - 用户id
     * @example
     * client.on("unmute-video", function(evt) {
     *    var uid = evt.uid;
     *    console.log("unmute video:" + uid);
     * });
     */
    /**
     * 该回调通知应用对方用户在语音通话中将自己的视频关闭。
     * @event RTCClient#event:mute-video
     * @type {object} evt
     * @property {string} uid - 用户id
     * @example
     * client.on("mute-video", function(evt) {
     *    var uid = evt.uid;
     *    console.log("mute video:" + uid);
     * });
     */
    /**
     * 该回调通知本地用户的角色发生改变。
     * @event RTCClient#event:client-role-changed
     * @type {object} evt
     * @property {role} - 角色，'host'或'audience'
     * @example
     * client.on("client-role-changed", function(evt) {
     *     console.log("client-role-changed", evt.role);
     * });
     */
    /**
     * 该回调通知应用本地音视频流已发布。
     * @event RTCClient#event:stream-published
     * @type {object} evt
     * @property {string} type - 事件类型, 值为`stream-published`
     * @property {string} proxy - 是否为代理
     * @property {Stream} stream - 本地流
     * @example
     * client.on("stream-published", function(evt) {
     *   console.log('local stream published');
     *   //……
     * });
     */
    /**
     * 该回调通知应用本地音视频流已取消发布。
     * @event RTCClient#event:stream-unpublished
     * @type {object} evt
     * @property {string} type - 事件类型，值为`stream-unpublished`
     * @property {Stream} stream - 本地流
     * @example
     * client.on("stream-unpublished", function(evt) {
     *   console.log('local stream unpublished');
     *   //……
     * });
     */
    /**
     * 该回调通知应用已接收远端音视频流。
     * @event RTCClient#event:stream-subscribed
     * @type {object} evt
     * @property {string} type - 事件类型，值为`stream-subscribed`
     * @property {string} msg - 消息
     * @property {string} reason - 原因
     * @property {Stream} stream - 远端流
     * @example
     * client.on("stream-subscribed", function(evt) {
     *   var stream = evt.stream;
     *   console.log("new stream subscribed ", stream.getId());
     *   // Play the stream.
     *   //……
     * });
     */
    /**
     * 该回调表示订阅流的音视频轨道发生变化。
     * @event RTCClient#event:stream-updated
     * @type {object} evt
     * @property {string} type - 事件类型，值为`stream-updated`
     * @property {Stream} stream - 远端流
     * @property {boolean} video - 是否有视频
     * @property {boolean} audio - 是否有音频
     */
    /**
     * 该回调表示远端流的网络统计情况，仅 Chromium/Chrome 内核支持比较完整
     * @event RTCClient#event:media-stats-update
     * @type {object} evt
     * @property {string} type - 事件类型，值为`media-stats-update`
     * @property {Stream} stream - 流
     * @property {RemoteStreamStats} statObj - 统计结果
     * @property {RemoteStreamStats} 详细字段
     * @property {audioReceiveBytes}: Bytes of the received audio.
     * @property {audioReceivePackets}: Number of packets of the received audio.
     * @property {audioReceivePacketsLost}: Number of lost packets of the received audio.
     * @property {videoReceiveBytes}: Bytes of the received video
     * @property {videoReceivePackets}: Packets of the received video
     * @property {videoReceivePacketsLost}: Number of lost packets of the received video
     * @property {videoReceiveResolutionWidth}: Resolution width of the received video
     * @property {videoReceiveResolutionHeight}: Resolution height of the received video
     * @property {videoReceiveDelay}: Delay in receiving the video (ms).
     * @property {audioReceiveDelay}: Delay in receiving the audio (ms).
     */
    /**
     * 该回调表示本地流的网络统计情况，仅 Chromium/Chrome 内核支持比较完整
     * @event RTCClient#event:local-media-stats-update
     * @type {object} evt
     * @property {string} type - 事件类型，值为`media-stats-update`
     * @property {LocalStreamStats} statObj - 统计结果
     * @property {LocalStreamStats} 详细字段
     * @property {audioSendBytes}: Bytes of the sent audio.
     * @property {audioSendPackets}: Number of packets of the sent audio.
     * @property {audioSendPacketsLost}: Number of lost packets of the sent audio.
     * @property {audioRtt}: Rtt of the sent audio.
     * @property {videoSendBytes}: Bytes of the sent video
     * @property {videoSendPackets}: Packets of the sent video
     * @property {videoSendPacketsLost}: Number of lost packets of the sent video
     * @property {videoSendResolutionWidth}: Resolution width of the sent video
     * @property {videoSendResolutionHeight}: Resolution height of the sent video
     * @property {accessDelay}: Delay in sent (ms).
     */
    /**
     * 该回调通知应用SDK 与服务器的连接状态发生改变
     *
     * SDK 与服务器的连接状态共有以下 4 种：
     * - DISCONNECTED：连接断开。该状态表示 SDK 处于以下任一阶段：
     *   - 调用 `client.join` 加入频道前的初始化阶段
     *   - 调用 `client.leave` 后的离开频道阶段
     * - CONNECTING：正在连接中。在调用 `client.join` 或者连接中断自动重连的时候为此状态。
     * - CONNECTED：已连接。该状态表示用户已经加入频道，可以在频道内发布或订阅媒体流。如果因
     * 网络断开或切换而导致 SDK 与服务器的连接中断，SDK 会自动重连，此时应用会收到该回调，通知网络状态由 CONNECTED 变为 CONNECTING。
     * - DISCONNECTING：正在断开连接。在调用 Client.leave 的时候为此状态。
     * @event RTCClient#event:connection-state-change
     * @type {object} evt
     * @property {string} curState - 当前的状态
     * @example
     * client.on("connection-state-change", function(evt) {
     *     console.log(evt.curState);
     * });
     */
    /**
     * 该回调通知正在与服务器重新建立连接。
     * @event RTCClient#event:reconnect
     * @type {object} evt
     * @property {string} type - 事件类型，值为`reconnect`
     * @example
     * client.on("reconnect", function() {
     *     console.log('reconnect');
     * });
     */
    /**
     * 该回调通知与服务器已经成功建立连接。
     * @event RTCClient#event:connected
     * @type {object} evt
     * @property {string} type - 事件类型，值为`connected`
     * @example
     * client.on("connected", function() {
     *    console.log('connected');
     * });
     */
    /**
     * 该回调通知应用有错误信息，需要进行处理，可能的错误如下：
     * - reason 为 "SOCKET_DISCONNECTED" 表示此时 SDK 重连失败，需要重新进入频道。
     * - reason 为其他错误，表示在连接断开后 SDK 自动重连阶段触发的错误。
     * @event RTCClient#event:error
     * @type {object} evt
     * @property {string} type - 事件类型，值为`error`
     * @example
     * client.on("error", function(err) {
     *   console.log("Got error msg:", err.reason);
     * });
     */
    private _attachSubControlEvents;
    private _attachRoomEvent;
    private _attachPublisherEvents;
    private _attachManagerEvents;
    private _dettachAllEvents;
}

interface HarkOption {
    smoothing?: number | undefined;
    interval?: number | undefined;
    threshold?: number | undefined;
    play?: boolean | undefined;
    history?: number | undefined;
    audioContext?: AudioContext | undefined;
}
declare class Hark extends WildEmitter {
    private audioContext;
    speaking: boolean;
    suspend: Function;
    resume: Function;
    setThreshold: Function;
    setInterval: Function;
    stop: Function;
    speakingHistory: Array<any>;
    constructor(stream: MediaStream, options: HarkOption);
}

/**
 * @typedef {Object} ClientConfig
 * @desc 定义 createClient 中的 config 参数的接口。
 * @param {string} mode - 模式: 'rtc'或'live'
 * @param {string} codec - 编码方式： 'vp8','h264'或'h264,vp8', 推荐默认填'h264,vp8'，兼容性最好
 * @param {string} server - 服务器地址，比如：'wss://xxxx.com'，需与youme确认是否支持
 * @param {boolean} [debug=false] - 是否开启调试模式，调试模式会输出内部日志；默认false
 */
/**
 * @typedef {Object} StreamSpec
 * @desc 创建流的配置参数
 * @param {number|string} [streamID=''] - 请设置streamId 为用户ID, 用户id可以从{@link RTCClient.join}返回值获取到
 * @param {boolean} audio - 表示流中是否包含音频轨道
 * @param {boolean} video - 表示流中是否包含视频轨道
 * @param {boolean} [screen] - 表示是否包含屏幕共享轨道
 * @param {boolean} [screenAudio=false] -
 *  `true`: 共享时共享本地正在播放的声音
 *  `false`: 默认值，共享时不要共享本地播放的声音
 */
/**
 * YMRTC
 *
 * @classdesc 游密Webrtc SDK
 */
declare class YMRTC extends WildEmitter {
    /**
     * @constant {string} - 版本号
     * @static
     */
    static VERSION: string;
    /**
     * 该方法用于创建客户端，在每次会话里仅调用一次。
     *
     * @param {ClientConfig} config - 定义客户端的属性
     * @return {RTCClient}
     * @example YMRTC.createClient(config)
     */
    static createClient(config: ClientConfig): RTCClient;
    static getScreenSources(callback: (sources: DesktopCapturerSource[]) => void): void;
    /**
     * 该方法创建并返回音视频流对象。
     *
     * @param {StreamSpec} spec - 定义音视频流对象的属性，详见 {StreamSpec}。
     * @return {Stream} - 流对象
     */
    /**
     * 该方法检查 Web SDK 对正在使用的浏览器的适配情况。
     * 你需要在创建音视频对象 (createClient)之前调用该方法，用以检查 Web SDK 对正在使用的浏览器的适配情况
     * 浏览器类型和版本均会影响返回结果,部分 Chromium 内核的浏览器（如 QQ 浏览器等），我们未做全量测试，如需使用，可以进行尝试。
     * @returns {boolean} - 是否支持
     * @example
     * const support = YMRTC.checkSystemRequirements()
     */
    static checkSystemRequirements(): boolean;
    static getSupportedCodec(supportedCodecOptions?: {
        stream: MediaStream;
    }): Promise<{
        video: Array<'VP8' | 'VP9' | 'H264'>;
        audio: Array<'OPUS'>;
    }>;
    /**
     * getDevices 成功回调
     *
     * @callback GetDevicesSuccCb
     * @param {MediaDeviceInfo[]} - devices 设备类型
     * @return {void}
     */
    /**
     * getDevices 失败回调
     *
     * @callback GetDevicesFailCb
     * @param {string} errStr - 错误原因
     * @return {void}
     */
    /**
     * 该方法枚举可用的媒体输入/输出设备，比如麦克风、摄像头、耳机等。
     * 调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的媒体设备以及设备信息。
     * @param {GetDevicesSuccCb} callback - 成功回调
     * @param {GetDevicesFailCb} [callback_Error] - 失败回调
     * @returns {void}
     */
    static getDevices(callback: (devices: MediaDeviceInfo[]) => void, callback_Error?: (errStr: string) => void): void;
    /**
     * 该方法枚举可用的媒体输入/输出音频设备，比如麦克风、耳机等。
     * 调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的媒体设备以及设备信息。
     *
     * @param {GetDevicesSuccCb} callback - 成功回调
     * @param {GetDevicesFailCb} [callback_Error] - 失败回调
     * @returns {void}
     */
    static getAudioDevices(callback: (devices: MediaDeviceInfo[]) => void, callback_Error?: (errStr: string) => void): void;
    /**
     * 该方法创建音频事件监听
     * 调用成功后返回Hark对象，可监听：
     * .on("speaking",()=>{开始讲话})
     * .on("stopped_speaking",()=>{停止讲话})
     * .on("volume_change",(volume,threshold)=>{音量变化}) , 音量范围范围：0.0到1.0
     * stop() 结束监听
     *
     * @param {Stream} stream - 绑定的流
     * @param {options} HarkOption - 只为监听讲话事件和音量，传{}即可
     * @returns {Hark}
     */
    static createHark(stream: any, options: HarkOption): Hark;
}

export { YMRTC as default };
